import datetime
import xmlsec

from django.conf import settings
from lxml import etree


def add_valid_until(xml_stream, dt_start):
    #template = etree.parse(xml_fname).getroot()
    #template = etree.fromstring(xml_stream).getroot()
    template = etree.XML(xml_stream)
    if template.get('validUntil'):
        return etree.tostring(template)

    offset = datetime.timedelta(minutes=settings.METADATA_VALID_UNTIL)
    dt = datetime.datetime.fromtimestamp(dt_start) + offset
    template.set('validUntil', dt.strftime("%Y-%m-%dT%H:%M:%SZ"))
    return etree.tostring(template)


def sign_xml(xml_stream, key_fname, cert_fname):
    template = etree.XML(xml_stream)

    opts = dict()
    if hasattr(template, 'getroot') and \
       hasattr(template.getroot, '__call__'):
        root = template.getroot()
    else:
        root = template
    idattr = root.get('ID') or root.get('id')
    if idattr:
        opts['reference_uri'] = "#{}".format(idattr)

    signed_xml = xmlsec.sign(root, key_fname, cert_fname, **opts)
    return etree.tostring(signed_xml)



from django.urls import path

from . import views

app_name = 'saml2_mdq'


urlpatterns = [
    path('entities/', views.saml2_entities, name='saml2_entities'),
    path('entities/<path:entity>', views.saml2_entity, name='saml2_entity'),
]



import base64
import hashlib
import logging
import os
import time

from django.conf import settings
from django.views.decorators.cache import cache_control
from django.http import HttpResponse, FileResponse, Http404
from django.shortcuts import render

from . utils import sign_xml, add_valid_until


logger = logging.getLogger(__name__)


@cache_control(max_age=getattr(settings, 'METADATA_CACHE_CONTROL', 3600))
def saml2_entities(request):
    md_path_file = settings.PYFF_METADATA_LOADED
    if not os.path.exists(md_path_file):
        msg = '{} Path does not exist'.format(md_path)
        logger.error(msg)
        return HttpResponse('', status=404)

    md_xml = open(md_path_file, 'rb').read()
    dt_file_mod = os.path.getmtime(md_path_file)

    # if there validUntil configuration
    if getattr(settings, 'METADATA_VALID_UNTIL', None):
        md_xml = add_valid_until(md_xml, dt_file_mod)

    # test the existence of rsa keys for signing
    key_fname = getattr(settings, 'METADATA_SIGNER_KEY', None)
    cert_fname = getattr(settings, 'METADATA_SIGNER_CERT', None)
    if key_fname and cert_fname:
        md_xml = sign_xml(md_xml, key_fname, cert_fname)

    # response
    return HttpResponse(md_xml,
                        content_type='application/samlmetadata+xml',
                        charset='utf-8')


@cache_control(max_age=getattr(settings, 'METADATA_CACHE_CONTROL', 3600))
def saml2_entity(request, entity):
    md_path = settings.PYFF_METADATA_FOLDER
    if not os.path.exists(md_path):
        msg = '{} Path does not exist'.format(md_path)
        logger.error(msg)
        return HttpResponse('', status=404)
    # path traversal prevention
    if entity != entity.replace('..', '').\
                        replace('%2e%2e', '').\
                        replace('\.', '').\
                        replace('%5C.', '').\
                        replace('%2e%2e%2f', '').\
                        replace('%252e%252e', ''):
        msg = 'Error Path traversal prevention on {}'.format(entity)
        logger.error(msg)
        return HttpResponse('Some digits in the entityID are not permitted',
                            status=403)

    if entity[:8] == '{base64}':
        entity_name = base64.b64decode(entity[8:]).decode()
        sha_entity = hashlib.sha1(entity_name.encode()).hexdigest()
        entity = '{sha1}'+'{}'.format(sha_entity)

    # if requested in sha1 format
    if entity[:6] == '{sha1}':
        md_try = os.path.sep.join((md_path, entity[6:]))
    else:
        sha_entity = hashlib.sha1(entity.encode()).hexdigest()
        md_try = os.path.sep.join((md_path, sha_entity))

    md_try += '.xml'
    dt_file_mod = os.path.getmtime(md_try)
    if os.path.exists(md_try):
        md_xml = open(md_try, 'rb').read()

        # if there validUntil configuration
        if getattr(settings, 'METADATA_VALID_UNTIL', None):
            md_xml = add_valid_until(md_xml, dt_file_mod)

        # test the existence of rsa keys for signing
        key_fname = getattr(settings, 'METADATA_SIGNER_KEY', None)
        cert_fname = getattr(settings, 'METADATA_SIGNER_CERT', None)
        if key_fname and cert_fname:
            md_xml = sign_xml(md_xml, key_fname, cert_fname)

        # response
        content_type='application/samlmetadata+xml, application/xml, text/xml'
        response =  HttpResponse(md_xml,
                                 content_type=content_type,
                                 charset='utf-8')
        response["Last-Modified"] = time.ctime(dt_file_mod)
        return response
    else:
        raise Http404()



from setuptools import setup

_name = 'saml2_mdq'

def readme():
    with open('README.md') as f:
        return f.read()

setup(name=_name,
      version='0.6.2',
      zip_safe = False,
      description="Django SAML MDQ",
      long_description=readme(),
      long_description_content_type="text/markdown",
      classifiers=['Development Status :: 5 - Production/Stable',
                  'License :: OSI Approved :: BSD License',
                  'Programming Language :: Python :: 2',
                  'Programming Language :: Python :: 3'],
      url='https://github.com/UniversitaDellaCalabria/Django-MDQ',
      author='Giuseppe De Marco',
      author_email='giuseppe.demarco@unical.it',
      license='BSD',
      packages=[_name],
      install_requires=['django', 'pyff'],
     )
